#include <stdio.h>
#include <stdlib.h>

void swapBytes(void *outData, const void *inData, size_t nBytes, int nItems){
	int i,j;

	const unsigned char *inPtr;
	unsigned char *outPtr;
	inPtr=(unsigned char *)inData;
	outPtr=(unsigned char *)outData;
	//printf("%i %i\n",nItems,nBytes);

	for(i=0;i<nItems;i++){
		for(j=0;j<nBytes;j++){
			outPtr[j] = inPtr[nBytes -j -1];
			//printf("%i %i %i\n",i,j,(int)(((void *)&outPtr[j])-outData));
		}
		inPtr += nBytes; outPtr += nBytes;		
	}
}

size_t fwriteRev(const void *ptr, size_t size, size_t nmemb, FILE *stream){
	void *tmp;

	tmp = malloc(size*nmemb);
	if(!tmp)return -1;
		
	swapBytes(tmp,ptr,size,nmemb);	
	fwrite(tmp,size,nmemb,stream);
	free(tmp);
}

size_t freadRev(void *ptr, size_t size, size_t nmemb, FILE *stream){
	void *tmp;

	tmp = malloc(size*nmemb);
	if(!tmp)return -1;

	fread(tmp,size,nmemb,stream);

	swapBytes(ptr,tmp,size,nmemb);	
	free(tmp);
}


int bmfWrite(const char *fileName, int rows, int cols, double *rowHead,double *colHead, double *data){
	FILE *f;
	int i;
	double zero = 0;
	
	f=fopen(fileName,"wb");
	if(!f)return 0;

	if(cols == 1 || colHead == NULL){ // no colums header for 1D data
		
		cols++;
		fwriteRev(&rows,4,1,f);
		fwriteRev(&cols,4,1,f);
		cols--;
		
	}else{
		//write number of cols/rows in file (including headers)
		cols++; rows++; 
		fwriteRev(&rows,4,1,f);
		fwriteRev(&cols,4,1,f);
		cols--; rows--; 
	
		//write the column headers (and a 0 that sits in the corner)
		fwriteRev(&zero,sizeof(double),1,f);
		fwriteRev(colHead,sizeof(double),cols,f);
	}

	for(i=0;i<rows;i++){
		fwriteRev(&rowHead[i],sizeof(double),1,f);
		fwriteRev(data + (i*cols),sizeof(double),cols,f);
	}

	fclose(f);

	return -1;
}

int bmfLoad(const char *fileName, int *rowsPtr, int *colsPtr, double **rowHeadPtr, double **colHeadPtr, double **dataPtr){
	FILE *f;
	int i;
	int rows,cols;
	double *rowHead;
	double *colHead;
	double *data;
	double zero;
	
	f=fopen(fileName,"rb");
	if(!f)return 0;

	freadRev(&rows,4,1,f);
	freadRev(&cols,4,1,f);
	cols--;
	if(cols > 1)rows--;
	
	rowHead=(double *)malloc(rows*sizeof(double));
	if(!rowHead)return 0;
	colHead=(double *)malloc(cols*sizeof(double));
	if(!colHead)return 0;
	data=(double *)malloc(rows*cols*sizeof(double));
	if(!data)return 0;

	if(cols <= 1)
		colHead[0] = 0;
	else{
		freadRev(&zero,sizeof(double),1,f);
		freadRev(colHead,sizeof(double),cols,f);
	}

	for(i=0;i<rows;i++){
		freadRev(&rowHead[i],sizeof(double),1,f);
		freadRev(data + (i*cols),sizeof(double),cols,f);
	}

	*rowsPtr = rows;
	*colsPtr = cols;
	*rowHeadPtr = rowHead;
	*colHeadPtr = colHead;
	*dataPtr = data;
	
	return -1;
}
