/*
 * KG4 Rewrite 
 *
 * Write out a new private KG4 PPF with reprocessed 
 * 
 *
*/

#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <cmath>
#include <cstdio>
#include <stdexcept>
#include "ppfc++.hpp"

int bmfLoad(const char *fileName, int *rows, int *cols, double **rowHead, double **colHead, double **data);
int bmfWrite(const char *fileName, int rows, int cols, double *rowHead,double *colHead, double *data);

using namespace std;
using namespace ppf;

// replace entries 4-8 of dtype with those in cacheFile
void loadCalib(string cacheFile, datatype &dtype){
	int nRows,nCols;
	double *rowHead, *colHead, *data;
		
	if(!bmfLoad(cacheFile.c_str(), &nRows, &nCols, &rowHead, &colHead, &data))
		throw runtime_error("bmfLoad failed for '"+cacheFile+"'");
	cout << "read " << nCols << " cols and " << nRows << " rows from " << cacheFile << "\n";

	for(int ch=4;ch<8;ch++){
		cout << "replacing calib " << dtype.getDataIt(ch) << " with " << data[ch] << "";
		dtype.setDataIt(ch, (float)(data[ch]) );		
	}
		
	if(rowHead)free(rowHead);
	if(colHead)free(colHead);
	if(data)free(data);
}

void replaceSig(string cacheFile, datatype &dtype){
int nRows,nCols;
	double *rowHead, *colHead, *data;
		
	if(!bmfLoad(cacheFile.c_str(), &nRows, &nCols, &rowHead, &colHead, &data))
		throw runtime_error("bmfLoad failed for '"+cacheFile+"'");
	cout << "read " << nCols << " cols and " << nRows << " rows from " << cacheFile << "\n";

	cout << "dtype originall: " << dtype.sizeT() << " in T and " << dtype.sizeX() << " in X\n";

	dtype.resizeTX(nRows, 1); //specifically 1 column
	for(int i=0;i<nRows;i++){
		dtype.setT(i,(float)(rowHead[i]));
		dtype.setDataIt(i,(float)(data[i]));
	}
	
		
	if(rowHead)free(rowHead);
	if(colHead)free(colHead);
	if(data)free(data);
}

int main(int argc, char *argv[])
{
	long shot;
		
 	shot = 70548; //atol(argv[2]);
 	
 	// 2nd version of code: verbose (explicit) version 
 	try
 	{
 		ppfdb.connect("JETPPF", ppf::read);
 		ppfdb.connect("oford", ppf::write);
 
		//geometry information - direct copy
		datatype R,Z,A,RREF,ZREF,AREF;
 		ppfdb.read(R, shot, "KG4", "R");
		ppfdb.read(Z, shot, "KG4", "Z");
		ppfdb.read(A, shot, "KG4", "A");
		ppfdb.read(RREF, shot, "KG4", "RREF");
		ppfdb.read(ZREF, shot, "KG4", "ZREF");
		ppfdb.read(AREF, shot, "KG4", "AREF");

		//previous calibration info
		datatype CCC,SHOT,THP0,PHI,XI,C,D;
		ppfdb.read(CCC, shot, "KG4", "CCC");
		ppfdb.read(SHOT, shot, "KG4", "SHOT");
		ppfdb.read(THP0, shot, "KG4", "THP0");
		ppfdb.read(PHI, shot, "KG4", "PHI");
		ppfdb.read(XI, shot, "KG4", "XI");
		ppfdb.read(C, shot, "KG4", "C");
		ppfdb.read(D, shot, "KG4", "D");


		//read FAR and ELL for all channels
		datatype FAR[8],ELL[8];
		for(int ch=0;ch<8;ch++){
			stringstream ss1,ss2;
			ss1 << "FAR" << (ch+1); ppfdb.read(FAR[ch], shot, "KG4", ss1.str());
			ss2 << "ELL" << (ch+1); ppfdb.read(ELL[ch], shot, "KG4", ss2.str());			
		}

		//mark laterals as reprocessed
		for(int ch=4;ch<8;ch++){
			SHOT.setDataIt(ch,shot);
			CCC.setDataIt(ch,999);
		}

		//load the reprocessed calibs
		stringstream dataRoot;
		dataRoot << "/work/data/" << shot << "/ppf-oford/kg4/";
	
		loadCalib(dataRoot.str() + "thp0", THP0);
		loadCalib(dataRoot.str() + "phi", PHI);
		loadCalib(dataRoot.str() + "xi", XI);
		loadCalib(dataRoot.str() + "c", C);
		loadCalib(dataRoot.str() + "d", D);

		//now load reproceesed FAR/ELL
		for(int ch=4;ch < 8;ch++){
			stringstream ss1,ss2;
			ss1 << dataRoot.str() << "far" << (ch+1);
			replaceSig(ss1.str(), FAR[ch]);
			ss2 << dataRoot.str() << "ell" << (ch+1);
			replaceSig(ss2.str(), ELL[ch]);
		}


		cout << "Made it to write!\n";

		//and write it all back
		ppfdb.create(shot, 1, "KG4 PPF - FAR/ELL 4-8 after osc removal and recalibration");
		cout << "Create OK\n";
		//geom and 5 calib parameters are per channel, so can share X-vectors and have no t-vectors
		ppfdb.write(R, "KG4", "R", ppf::tvector);
 		ppfdb.write(Z, "KG4", "Z", ppf::lasttvector);
 		ppfdb.write(A, "KG4", "A", ppf::lasttvector);
 		ppfdb.write(RREF, "KG4", "RREF", ppf::lasttvector);
 		ppfdb.write(ZREF, "KG4", "ZREF", ppf::lasttvector);
 		ppfdb.write(AREF, "KG4", "AREF", ppf::lasttvector);
		cout << "Block 1 OK\n";
 		
		ppfdb.write(CCC, "KG4", "CCC",  ppf::lasttvector);
		ppfdb.write(SHOT, "KG4", "SHOT",  ppf::lastxvector);
		ppfdb.write(THP0, "KG4", "THP0",  ppf::lasttvector);
		ppfdb.write(PHI, "KG4", "PHI",  ppf::lasttvector);
		ppfdb.write(XI, "KG4", "XI",  ppf::lasttvector);
		ppfdb.write(C, "KG4", "C",  ppf::lasttvector);
		ppfdb.write(D, "KG4", "D",  ppf::lasttvector);
		cout << "Block 2 OK\n";
 		
		
		for(int ch=0;ch<8;ch++){
			//far and ell can share t-vector, but we use a different one for each channel
			//since channels can be missing and the laterals now might be real-time
			stringstream ss1,ss2;
			ss1 << "FAR" << (ch+1);
			ppfdb.write(FAR[ch], "KG4", ss1.str(), ppf::tvector);
			ss2 << "ELL" << (ch+1);
			ppfdb.write(ELL[ch], "KG4", ss2.str(), ppf::lasttvector);
		}
		cout << "Block 3 OK\n";
 		
		
		ppfdb.closeDDA(1, "KG4 PPF - FAR/ELL 4-8 after osc removal and recalibration");
 		ppfdb.commit("rmOsc", 1);
		cout << "Close and commit OK\n";

 		
/*
		//Reading code 

		double colHead[] = { 0, 0 };
 		double *rowHead;
 		double *data;
 	
 		rowHead = (double *)malloc(sizeof(double) * a.sizeT());
 		data = (double *)malloc(sizeof(double) * a.sizeT());
 		if(!rowHead || !data){ printf("Malloc fail\n"); return 1; }

 		for(int i=0;i<a.sizeT();i++){
 			rowHead[i] = a.getT(i);
 			data[i] = a.getDataIt(i);
			printf("%i %f %f\n",i,rowHead[i],data[i]);
 		}

  		bmfWrite((char *)"./merp.bin",a.sizeT(),1,rowHead,NULL,data);
*/
 	}
 	catch (ppf_error &e)
 	{
 		cout << e.message << " Code=" << e.code << ", Routine=" << e.method << endl;
 		exit(1);
 	}
	catch (runtime_error &e)
	{
		cout << e.what();
 		exit(1);
 	}
 	
   	return 0;
}



